@@ -432,6 +432,28 @@ def compatibility_scores(h1, h2, dataset, device="cpu"):
     return btc_dataset, bec_dataset
 
 
+def model_accuracy(model, dataset, device="cpu"):
+    number_of_batches = len(dataset)
+    model_performance = 0
+    with torch.no_grad():
+        for data, target in dataset:
+            if device != "cpu":
+                data = data.to(device)
+                target = target.to(device)
+            _, _, output_logsoftmax = model(data)
+            output_labels = torch.argmax(output_logsoftmax, 1)
+            if device != "cpu":
+                output_labels = output_labels.cpu()
+                target = target.cpu()
+            performance = accuracy_score(output_labels.numpy(), target.numpy())
+            model_performance += performance
+            # _clean_from_gpu([data, target])
+
+        model_performance /= number_of_batches
+
+    return model_performance
+
+
 def evaluate_model_performance_and_compatibility_on_dataset(h1, h2, dataset, performance_metric=None,
                                                             device="cpu"):
     """
@@ -563,7 +585,7 @@ def train_new_error(h1, h2, number_of_epochs,
                     training_set, test_set, batch_size_train, batch_size_test,
                     OptimizerClass, optimizer_kwargs,
                     NewErrorLossClass,
-                    lambda_c, device="cpu"):
+                    lambda_c, new_error_loss_kwargs=None, device="cpu"):
     """
     Args:
         h1: Reference Pytorch model.
@@ -587,7 +609,7 @@ def train_new_error(h1, h2, number_of_epochs,
             to set this to "cuda". This makes sure that the input and target
             tensors are transferred to the GPU during training.
     """
-    bc_loss = NewErrorLossClass(h1, h2, lambda_c)
+    bc_loss = NewErrorLossClass(h1, h2, lambda_c, **new_error_loss_kwargs)
     new_optimizer = OptimizerClass(h2.parameters(), **optimizer_kwargs)
     _, _, _, _ = train_compatibility(
         number_of_epochs, h2, new_optimizer, bc_loss, training_set, test_set,
@@ -598,7 +620,7 @@ def train_strict_imitation(h1, h2, number_of_epochs,
                            training_set, test_set, batch_size_train, batch_size_test,
                            OptimizerClass, optimizer_kwargs,
                            StrictImitationLossClass,
-                           lambda_c, device="cpu"):
+                           lambda_c, strict_imitation_loss_kwargs=None, device="cpu"):
     """
     Args:
         h1: Reference Pytorch model.
@@ -622,7 +644,7 @@ def train_strict_imitation(h1, h2, number_of_epochs,
             to set this to "cuda". This makes sure that the input and target
             tensors are transferred to the GPU during training.
     """
-    si_loss = StrictImitationLossClass(h1, h2, lambda_c)
+    si_loss = StrictImitationLossClass(h1, h2, lambda_c, **strict_imitation_loss_kwargs)
     new_optimizer = OptimizerClass(h2.parameters(), **optimizer_kwargs)
     _, _, _, _ = train_compatibility(
         number_of_epochs, h2, new_optimizer, si_loss, training_set, test_set,
@@ -635,6 +657,8 @@ def compatibility_sweep(sweeps_folder_path, number_of_epochs, h1, h2,
                         NewErrorLossClass, StrictImitationLossClass,
                         performance_metric=None,
                         lambda_c_stepsize=0.25, percent_complete_queue=None,
+                        new_error_loss_kwargs=None,
+                        strict_imitation_loss_kwargs=None,
                         device="cpu"):
     """
     This function trains a new model using the backward compatibility loss function
@@ -677,7 +701,14 @@ def compatibility_sweep(sweeps_folder_path, number_of_epochs, h1, h2,
     number_of_trainings = 4 * len(np.arange(0.0, 1.0 + (lambda_c_stepsize / 2), lambda_c_stepsize))
     if percent_complete_queue is not None:
         percent_complete_queue.put(0.0)
-    sweep_summary = []
+
+    if new_error_loss_kwargs is None:
+        new_error_loss_kwargs = dict()
+
+    if strict_imitation_loss_kwargs is None:
+        strict_imitation_loss_kwargs = dict()
+
+    sweep_summary_data = []
     datapoint_index = 0
     for lambda_c in np.arange(0.0, 1.0 + (lambda_c_stepsize / 2), lambda_c_stepsize):
         h2_new_error = copy.deepcopy(h2)
@@ -685,6 +716,7 @@ def compatibility_sweep(sweeps_folder_path, number_of_epochs, h1, h2,
             h1, h2_new_error, number_of_epochs,
             training_set, test_set, batch_size_train, batch_size_test,
             OptimizerClass, optimizer_kwargs, NewErrorLossClass, lambda_c,
+            new_error_loss_kwargs=new_error_loss_kwargs,
             device=device)
         h2_new_error.eval()
         torch.save(h2_new_error.state_dict(), f"{sweeps_folder_path}/{lambda_c}-model-new-error.state")
@@ -697,7 +729,7 @@ def compatibility_sweep(sweeps_folder_path, number_of_epochs, h1, h2,
         training_set_performance_and_compatibility["training"] = True
         training_set_performance_and_compatibility["testing"] = False
         training_set_performance_and_compatibility["datapoint_index"] = datapoint_index
-        sweep_summary.append({
+        sweep_summary_data.append({
             "datapoint_index": datapoint_index,
             "lambda_c": lambda_c,
             "training": True,
@@ -722,7 +754,7 @@ def compatibility_sweep(sweeps_folder_path, number_of_epochs, h1, h2,
         testing_set_performance_and_compatibility["training"] = False
         testing_set_performance_and_compatibility["testing"] = True
         testing_set_performance_and_compatibility["datapoint_index"] = datapoint_index
-        sweep_summary.append({
+        sweep_summary_data.append({
             "datapoint_index": datapoint_index,
             "lambda_c": lambda_c,
             "training": False,
@@ -744,6 +776,7 @@ def compatibility_sweep(sweeps_folder_path, number_of_epochs, h1, h2,
             h1, h2_strict_imitation, number_of_epochs,
             training_set, test_set, batch_size_train, batch_size_test,
             OptimizerClass, optimizer_kwargs, StrictImitationLossClass, lambda_c,
+            strict_imitation_loss_kwargs=strict_imitation_loss_kwargs,
             device=device)
         h2_strict_imitation.eval()
         torch.save(h2_strict_imitation.state_dict(), f"{sweeps_folder_path}/{lambda_c}-model-strict-imitation.state")
@@ -756,7 +789,7 @@ def compatibility_sweep(sweeps_folder_path, number_of_epochs, h1, h2,
         training_set_performance_and_compatibility["training"] = True
         training_set_performance_and_compatibility["testing"] = False
         training_set_performance_and_compatibility["datapoint_index"] = datapoint_index
-        sweep_summary.append({
+        sweep_summary_data.append({
             "datapoint_index": datapoint_index,
             "lambda_c": lambda_c,
             "training": True,
@@ -781,7 +814,7 @@ def compatibility_sweep(sweeps_folder_path, number_of_epochs, h1, h2,
         testing_set_performance_and_compatibility["training"] = False
         testing_set_performance_and_compatibility["testing"] = True
         testing_set_performance_and_compatibility["datapoint_index"] = datapoint_index
-        sweep_summary.append({
+        sweep_summary_data.append({
             "datapoint_index": datapoint_index,
             "lambda_c": lambda_c,
             "training": False,
@@ -801,6 +834,11 @@ def compatibility_sweep(sweeps_folder_path, number_of_epochs, h1, h2,
         if percent_complete_queue is not None:
             percent_complete_queue.put((datapoint_index) / number_of_trainings)
 
+    sweep_summary = {
+        "data": sweep_summary_data,
+        "h1_performance": model_accuracy(h1, test_set)
+    }
+
     sweep_summary_data = json.dumps(sweep_summary)
     sweep_summary_data_file = open(f"{sweeps_folder_path}/sweep_summary.json", "w")
     sweep_summary_data_file.write(sweep_summary_data)
